@@ -1,8 +1,8 @@
 from __future__ import unicode_literals
 
-from django.conf.urls import url
+from django.conf.urls import include, url
 
-from rest_framework.compat import RegexURLResolver, include
+from rest_framework.compat import RegexURLResolver, get_regex_pattern
 from rest_framework.settings import api_settings
 
 
@@ -11,7 +11,7 @@ def apply_suffix_patterns(urlpatterns, suffix_pattern, suffix_required):
     for urlpattern in urlpatterns:
         if isinstance(urlpattern, RegexURLResolver):
             # Set of included URL patterns
-            regex = urlpattern.regex.pattern
+            regex = get_regex_pattern(urlpattern)
             namespace = urlpattern.namespace
             app_name = urlpattern.app_name
             kwargs = urlpattern.default_kwargs
@@ -19,11 +19,10 @@ def apply_suffix_patterns(urlpatterns, suffix_pattern, suffix_required):
             patterns = apply_suffix_patterns(urlpattern.url_patterns,
                                              suffix_pattern,
                                              suffix_required)
-            ret.append(url(regex, include(patterns, namespace, app_name), kwargs))
-
+            ret.append(url(regex, include((patterns, app_name), namespace), kwargs))
         else:
             # Regular URL pattern
-            regex = urlpattern.regex.pattern.rstrip('$').rstrip('/') + suffix_pattern
+            regex = get_regex_pattern(urlpattern).rstrip('$').rstrip('/') + suffix_pattern
             view = urlpattern.callback
             kwargs = urlpattern.default_args
             name = urlpattern.name
